'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var http = _interopDefault(require('http'));
var https = _interopDefault(require('https'));
var FormData = _interopDefault(require('form-data'));
var path$1 = require('path');
var fs = require('fs');
var stream = require('stream');

const parse = url => {
  const _url = new URL(url);

  const request = _url.protocol === 'http:' ? http.request : https.request;
  return [_url, request];
};

function post(url, data) {
  const [_url, request] = parse(url);
  return new Promise((resolve, reject) => {
    const req = request(_url, {
      method: 'POST',
      headers: data.getHeaders(),
      ...this
    });
    req.on('error', reject);
    req.on('response', res => {
      if (res.statusCode === 200) {
        resolve(res);
      } else {
        res.resume();
        reject(new Error(res.statusCode + ' ' + res.statusMessage));
      }
    });
    data.pipe(req);
  });
}
function get(url) {
  const [_url, request] = parse(url);
  return new Promise((resolve, reject) => {
    const req = request(_url, {
      method: 'GET',
      ...this
    });
    req.on('error', reject);
    req.on('response', res => {
      if (res.statusCode === 200) {
        resolve(res);
      } else {
        res.resume();
        reject(new Error(res.statusCode + ' ' + res.statusMessage));
      }
    });
    req.end();
  });
}

const client = config => ({
  post: post.bind(config || null),
  get: get.bind(config || null)
});

function gotenberg(url, client$1, config) {
  let clnt;

  if (typeof client$1 === 'object' && 'post' in client$1) {
    clnt = client$1;
  } else if (typeof client$1 === 'function') {
    if (/^class\s/.test(Function.prototype.toString.call(client$1))) {
      clnt = new client$1(config);
    } else {
      clnt = client$1(config);
    }
  } else {
    clnt = client(config || client$1);
  }

  return source => ({
    type: 6,
    url: url.toString(),
    client: clnt,
    source,
    fields: {}
  });
}

const isString = x => typeof x === 'string';
const isBuffer = x => x != null && x instanceof Buffer;
const isStream = x => x != null && x instanceof stream.Readable;
const isURL = x => x != null && x instanceof URL;
const isFileUri = x => isString(x) && x.startsWith('file:');
const isPlain = x => isString(x) || isStream(x) || isBuffer(x);
const isTuple = x => Array.isArray(x) && x.length === 2 && typeof x[0] === 'string' && isPlain(x[1]);
const isObject = x => {
  if (x == null || typeof x !== 'object' || Array.isArray(x) || typeof x[Symbol.iterator] === 'function' || x instanceof URL) {
    return false;
  }

  for (const key in x) {
    if (x.hasOwnProperty(key) && !isPlain(x[key])) return false;
  }

  return true;
};
const isIterable = x => {
  if (x == null || typeof x === 'string') return false;

  if (typeof x[Symbol.iterator] === 'function') {
    for (const src of x) {
      if (src == null) return false;
      if (!isPlain(src) && !isTuple(src) && !isObject(src)) return false;
    }

    return true;
  }

  return false;
};
const fileNameRE = /^[\w\s\(\),-]+\.[A-Za-z0-9]+$/;
const MIN_FILE_NAME_LENGTH = 3;
const MAX_FILE_NAME_LENGTH = 50;
const isFileName = x => isString(x) && !x.startsWith('file:') && x.length >= MIN_FILE_NAME_LENGTH && x.length <= MAX_FILE_NAME_LENGTH && fileNameRE.test(x);

const toTuples = (source, recursive = false) => {
  if (isURL(source)) return [];

  if (isFileUri(source)) {
    return !recursive && path$1.extname(source) === '.html' ? [['index.html', source]] : [[path$1.basename(source), source]];
  }

  if (isString(source) || isBuffer(source)) {
    return [['index.html', source]];
  }

  if (isStream(source)) {
    if (source instanceof fs.ReadStream) {
      const name = path$1.basename(String(source.path));
      return !recursive && path$1.extname(name) === '.html' ? [['index.html', source]] : [[name, source]];
    } else {
      return [['index.html', source]];
    }
  }

  if (isTuple(source) && isFileName(source[0])) {
    return [source];
  }

  if (isObject(source)) {
    const ret = [];

    for (const key in source) {
      if (source.hasOwnProperty(key)) {
        ret.push([key, source[key]]);
      }
    }

    return ret;
  }

  if (recursive) {
    throw new Error('Bad source, possible recursive iterables?');
  }

  if (isIterable(source)) {
    const ret = [];

    for (const src of source) {
      ret.push(...toTuples(src, true));
    }

    return ret;
  }

  throw new Error(`Bad source, don't know what to do with "${source}"`);
};
const fromFile = path => fs.createReadStream(path.replace(/^file:(\/\/)?/, ''));
const toStream = source => isStream(source) ? source : isFileUri(source) ? fromFile(source) : new stream.Readable({
  read() {
    this.push(source);
    this.push(null);
  }

});
const toStreams = source => {
  if (!source) return [];
  const tuples = toTuples(source);
  const ret = [];

  for (let i = 0; i < tuples.length; i++) {
    ret.push([tuples[i][0], toStream(tuples[i][1])]);
  }

  return ret;
};

const formdata = (fields, files) => {
  const data = new FormData();

  for (const field in fields) {
    if (fields.hasOwnProperty(field)) {
      const value = fields[field];

      if (value !== undefined) {
        data.append(field, String(value));
      }
    }
  }

  for (let i = 0; i < files.length; i++) {
    const file = files[i];
    data.append(file[0], file[1], {
      filename: file[0]
    });
  }

  return data;
};

function please(request) {
  if (request.type === 1) {
    if (typeof request.client.get === 'function') {
      return request.client.get(request.url).then(response => {
        response.resume();
      });
    }

    throw new Error(`Gotenberg client doesn't implements "get" method`);
  }

  const sources = toStreams(request.source);
  const form = formdata(request.fields, sources);
  return request.client.post(request.url, form);
}

function pipe(...fns) {
  return source => fns.reduce((request, fn) => fn(request), source);
}

function fields(fields) {
  return request => ({ ...request,
    fields: { ...request.fields,
      ...fields
    }
  });
}

function path(path) {
  return request => ({ ...request,
    url: (request.url || '') + path
  });
}

function type(type) {
  return request => ({ ...request,
    type
  });
}

function markdown(x) {
  return 'type' in x ? pipe(path('/markdown'), type(5))(x) : pipe(fields(x), path('/markdown'), type(5));
}

var convert = path('/convert');

function office(request) {
  return pipe(path('/office'), type(4))(request);
}

function merge(request) {
  if (!isIterable(request.source) && !isObject(request.source)) {
    throw new Error('Invalid source, should be iterable or object');
  }

  return pipe(path('/merge'), type(3))(request);
}

function ping(request) {
  return pipe(path('/ping'), type(1))(request);
}

function html(x) {
  return 'type' in x ? pipe(path('/html'), type(2))(x) : pipe(fields(x), path('/html'), type(2));
}

function url(request) {
  let remoteURL;
  if (isString(request.source)) remoteURL = request.source;else if (isURL(request.source)) remoteURL = request.source.toString();else {
    throw new Error('Invalid source, should be url string or instance of URL');
  }
  return pipe(fields({
    remoteURL,
    marginTop: 0,
    marginBottom: 0,
    marginLeft: 0,
    marginRight: 0
  }), path('/url'), type(0))({ ...request,
    source: undefined
  });
}

function set(...opts) {
  const options = {};

  for (let i = opts.length; i--;) {
    const op = opts[i];

    if (typeof op === 'function') {
      op(options);
    } else {
      Object.assign(options, op);
    }
  }

  return fields(options);
}

const A3 = [11.7, 16.5];
const A4 = [8.27, 11.7];
const A5 = [5.8, 8.3];
const A6 = [4.1, 5.8];
const LETTER = [8.5, 11];
const LEGAL = [8.5, 14];
const TABLOID = [11, 17];
const NO_MARGINS = [0, 0, 0, 0];
const NORMAL_MARGINS = [1, 1, 1, 1];
const LARGE_MARGINS = [2, 2, 2, 2];

const noop = () => undefined;

const landscape = fields => fields.landscape = true;
const portrait = fields => fields.landscape = undefined;
function paperSize(paper) {
  if (!paper) return noop;
  return fields => {
    if (Array.isArray(paper)) {
      [fields.paperWidth, fields.paperHeight] = paper;
    } else {
      ({
        width: fields.paperWidth,
        height: fields.paperHeight
      } = paper);
    }
  };
}
const a3 = paperSize(A3);
const a4 = paperSize(A4);
const a5 = paperSize(A5);
const a6 = paperSize(A6);
const legal = paperSize(LEGAL);
const letter = paperSize(LETTER);
const tabloid = paperSize(TABLOID);
function marginSizes(margins) {
  if (!margins) return noop;
  return fields => {
    if (Array.isArray(margins)) {
      [fields.marginTop, fields.marginRight, fields.marginBottom, fields.marginLeft] = margins;
    } else {
      ({
        top: fields.marginTop,
        right: fields.marginRight,
        bottom: fields.marginBottom,
        left: fields.marginLeft
      } = margins);
    }
  };
}
const noMargins = marginSizes(NO_MARGINS);
const normalMargins = marginSizes(NORMAL_MARGINS);
const largeMargins = marginSizes(LARGE_MARGINS);

function to(...opts) {
  const options = {};
  let paper;
  let margins;

  for (let i = opts.length; i--;) {
    const op = opts[i];

    if (typeof op === 'function') {
      op(options);
      continue;
    }

    if (Array.isArray(op)) {
      if (op.length === 2) paper = op;
      if (op.length === 4) margins = op;
      continue;
    }

    if ('width' in op || 'height' in op) {
      paper = op;
      continue;
    }

    if ('top' in op || 'right' in op || 'bottom' in op || 'left' in op) {
      margins = op;
      continue;
    }

    if ('paper' in op && op.paper) {
      paper = op.paper;
      op.paper = undefined;
    }

    if ('margins' in op && op.margins) {
      margins = op.margins;
      op.margins = undefined;
    }

    Object.assign(options, op);
  }

  paperSize(paper)(options);
  marginSizes(margins)(options);
  return fields(options);
}

const filename = name => fields => fields.resultFilename = name;
const timeout = timeout => fields => fields.waitTimeout = timeout;
const delay = delay => fields => fields.waitDelay = delay;
const webhook = (url, timeout) => fields => {
  fields.webhookURL = url;
  fields.webhookURLTimeout = timeout;
};
const googleChromeRpccBufferSize = googleChromeRpccBufferSize => fields => fields.googleChromeRpccBufferSize = googleChromeRpccBufferSize;

exports.A3 = A3;
exports.A4 = A4;
exports.A5 = A5;
exports.A6 = A6;
exports.LARGE_MARGINS = LARGE_MARGINS;
exports.LEGAL = LEGAL;
exports.LETTER = LETTER;
exports.NORMAL_MARGINS = NORMAL_MARGINS;
exports.NO_MARGINS = NO_MARGINS;
exports.TABLOID = TABLOID;
exports.a3 = a3;
exports.a4 = a4;
exports.a5 = a5;
exports.a6 = a6;
exports.convert = convert;
exports.delay = delay;
exports.filename = filename;
exports.googleChromeRpccBufferSize = googleChromeRpccBufferSize;
exports.gotenberg = gotenberg;
exports.html = html;
exports.landscape = landscape;
exports.largeMargins = largeMargins;
exports.legal = legal;
exports.letter = letter;
exports.marginSizes = marginSizes;
exports.markdown = markdown;
exports.merge = merge;
exports.noMargins = noMargins;
exports.normalMargins = normalMargins;
exports.office = office;
exports.paperSize = paperSize;
exports.ping = ping;
exports.pipe = pipe;
exports.please = please;
exports.portrait = portrait;
exports.set = set;
exports.tabloid = tabloid;
exports.timeout = timeout;
exports.to = to;
exports.url = url;
exports.webhook = webhook;
//# sourceMappingURL=index.js.map
